;LCD-Routinen fr WD-C2704M - Text-Punktmatrix-LCD
;mit 2 Controllern HD44780. 4 Zeilen je 27 Zeichen

;Version mit frei definierbaren Steuerleitungen.

;Diese Routinensammlung baut auf Beispielen von Andreas Schwarz
;und T.S. auf und ist von ...HanneS... zusammengestellt worden.
; hannes.lux@gmx.de / www.hannes.de.md

;============================================================================
;Diese Sammlung enthlt folgende Routinen:
;----------------------------------------------------------------------------
;  lcd_init             ;initialisiert und lscht beide Controller des LCD
;----------------------------------------------------------------------------
;  lcd_clear            ;lscht das LCD
;----------------------------------------------------------------------------
;  lcd_command          ;bergibt das Byte im Register "wl" als Kommando
                        ;an das LCD
;----------------------------------------------------------------------------
;  lcd_data             ;bergibt das Byte im Register "wl" zur Textausgabe
;----------------------------------------------------------------------------
;  locate			;Macro zum Einstellen der Ausgabeposition.
                        ;Aufruf: locate Zeile,Spalte
                        ;Zeile: 0-3, Spalte: 0-26
;============================================================================
;Die Datei "LCDprint.inc" stellt einige Ausgaberoutinen zur Verfgung, die
;die Ausgabe von Texten und Zahlen erleichtert. Zusammen mit "locate" wird
;die Ausgabe von Mentexten, Zahlen usw. hnlich einfach wie in BASIC.
;============================================================================
;Vor Schreibzugriffen wird das Busy-Flag des LCD ausgelesen und
;auf LCD-Bereitschaft gewartet.

;Fr die Auswahl des zustndigen Controllers werden die zwei Bits 
;"lcdcontroller1" und "lcdcontroller2" im Register "FLAGS"
;des Hauptprogramms genutzt. Sie mssen dort deklariert werden.
;Die bergabe der Daten erfolgt ber das Register "wl", das im Hauptprogramm
;zu deklarieren ist. Weiterhin muss das Register "wh" deklariert sein.
;============================================================================
;Anschlussbelegung:     ;Die 4 Datenleitungen sollten in einem Nibble liegen.
                        ;Dies kann ein unteres oder oberes Nibble sein.
                        ;Die Auswahl geschieht mit "lcddatmsk"
                        ;Die Steuerleitungen des LCD knnen ber alle
                        ;verfgbaren Ports frei gewhlt werden, was diesen
                        ;Treiber sehr flexibel macht.
;----------------------------------------------------------------------------
.equ lcdport=portc      ;Port, an dem die 4 Datenleitungen des LCD
                        ;angeschlossen sind.
.equ lcddatmsk=$0f      ;Maske auf LCD-Datenpins.
                        ;$0f - LCD ist an Bit 0...3 des Ports angeschlossen.
                        ;$f0 - LCD ist an Bit 4...7 des Ports angeschlossen.
;----------------------------------------------------------------------------
.equ rsport=portd       ;Port, an dem RS angeschlossen ist
.equ rs=5               ;Pin, an dem RS angeschlossen ist
;----------------------------------------------------------------------------
.equ enable1port=portd  ;Port Anschluss Enable des ersten Controllers.
.equ enable1=6          ;Pin Anschluss Enable des ersten Controllers.
;----------------------------------------------------------------------------
.equ enable2port=portd  ;Port Anschluss Enable des zweiten Controllers.
.equ enable2=7          ;Pin Anschluss Enable des zweiten Controllers.
;----------------------------------------------------------------------------
.equ rwport=portd       ;Port, an dem R/W angeschlossen ist
.equ rw=0               ;Pin, an dem R/W angeschlossen ist
;============================================================================


;=============================================================================
;  locate Zeile,Spalte  ;Setzt die Ausgabeposition des LCD auf
                        ;Zeile (0...3) und Spalte (0...26).
                        ;Dabei wird auch der fr die Ausgabe zustndige
                        ;Controller ausgewhlt.
;-----------------------------------------------------------------------------
.macro locate ;Zeile (0...3), Spalte (0...27)
                        ;Positionierung der Ausgabeposition
                            ;Beide Controller erstmal aus:
 cbr flags,(1<<lcdcontroller2)|(1<<lcdcontroller1)  
                            ;erforderlichen Controller aktivieren:
 sbr flags,((@0 & 2)<<(lcdcontroller2-1))|((!(@0 & 2))<<lcdcontroller1)
 push wl                    ;Register sichern
                            ;Zeile in Bit 6, Spalte in den Rest:
 ldi wl,128+((@0 & 1)<<6)+(@1 & 63)
 rcall lcd_command          ;an LCD als Befehl ausgeben
 pop wl                     ;Register wiederherstellen
.endmacro


;============================================================================
; Aufrufbare Routine, initialisiert und lscht das LCD
;----------------------------------------------------------------------------
lcd_init:               ;LCD-Initialisierung

 sbr flags,(1<<lcdcontroller1)|(1<<lcdcontroller2)
                            ;erstmal beide LCD-Controller aktivieren

 in wl,lcdport-1            ;Datenrichtung LCD-Datenport holen
 sbr wl,lcddatmsk           ;benutzte Datenpins als Ausgang
 out lcdport-1,wl           ;definieren
 sbi rsport-1,rs            ;RS-Leitung als Ausgang
 sbi enable1port-1,enable1  ;Enable1-Leitung als Ausgang
 sbi enable2port-1,enable2  ;Enable2-Leitung als Ausgang
 sbi rwport-1,rw            ;Read/Write-Leitung als Ausgang

 push xl                    ;Inhalt sichern
 ldi xl,50                  ;>15ms warten, nachdem Vcc > 4,5V ist
powerupwait:                ;hier 250ms warten, dafr keine Spannungsprfung
 rcall delay5ms
 dec xl
 brne powerupwait
 pop xl                     ;alten Inhalt zurck holen

 cbi rwport,rw              ;Steuerleitungen auf
 cbi rsport,rs              ;L-Pegel legen
;Startsequenz
 in wl,lcdport              ;ganzen Port einlesen
 cbr wl,lcddatmsk           ;Platz fr Datenbits lschen
 ldi wh,0b00110011          ;bentigte Datenbits in beide Nibbles
 andi wh,lcddatmsk          ;nur das genutzte Nibble
 or wl,wh                   ;bernehmen
 out lcdport,wl             ;und wieder ausgeben
;dreimal ausgeben
 rcall lcd_impuls           ;1. Ausgabe (von 3)
 rcall delay5ms             ;>4,1ms warten
 rcall lcd_impuls           ;2
 rcall delay5ms             ;>0,1ms warten
 rcall lcd_impuls           ;und 3!
 rcall delay5ms             ;Ab hier BUSY prfen
;4-Bit-Modus aktivieren
 in wl,lcdport              ;ganzen Port einlesen
 cbr wl,lcddatmsk           ;Platz fr Datenbits lschen
 ldi wh,0b00100010          ;bentigte Datenbits in beide Nibbles
 andi wh,lcddatmsk          ;nur das genutzte Nibble
 or wl,wh                   ;bernehmen
 out lcdport,wl             ;und wieder ausgeben
 rcall lcd_impuls
 rcall delay5ms
;ab jetzt 4-Bit-Zugriff mit Busy-Abfrage
 ldi wl,0b00101000          ;4bit-Modus einstellen
 rcall lcd_command

 ldi wl,0b00001100          ;Display ein, Cursor aus
 rcall lcd_command

 ldi wl,0b00000100          ;Entry-Mode
 rcall lcd_command
                            ;(Code geht weiter...)


;============================================================================
; Aufrufbare Routine, lscht das LCD
;----------------------------------------------------------------------------
lcd_clear:  ;Sendet den Befehl zur Lschung des Displays
 sbr flags,(1<<lcdcontroller1)|(1<<lcdcontroller2) ;beide Controller
 ldi wl,0b00000001          ;Display lschen
 rcall lcd_command
 cbr flags,1<<lcdcontroller2;nur ersten Controller aktiviert lassen
 rcall delay5ms
 ret


;============================================================================
; Aufrufbare Routine, sendet den Befehl in wl an das LCD
;----------------------------------------------------------------------------
lcd_command:            ;sendet den Befehl in wl an das LCD
 rcall lcd_busy             ;LCD auf Bereitschaft prfen
 push wh                    ;fr Nibble-Wahl
 ldi wh,lcddatmsk           ;Referenz fr Nibblewahl
 push wl                    ;Commandobyte sichern
 sbrc wh,0                  ;wenn LCD am unteren Nibble angeschlossen ist
 swap wl                    ;dann oberes Nibble nach unten
 cbi rsport,rs              ;RS lschen
 rcall lcd_nibble           ;erstes Nibble ausgeben...
 pop wl                     ;Kopie vom Commandobyte holen
 sbrc wh,4                  ;wenn LCD am oberen Nibble angeschlossen ist
 swap wl                    ;dann unteres Nibble nach oben
 pop wh                     ;wieder frei
 cbi rsport,rs              ;RS lschen
 rjmp lcd_nibble            ;zweites Nibble ausgeben und zurck...


;============================================================================
; Aufrufbare Routine, sendet das Zeichen in wl an das LCD
;----------------------------------------------------------------------------
lcd_data:               ;sendet das Zeichen in wl an das LCD
 rcall lcd_busy             ;LCD auf Bereitschaft prfen
 
 push wh                    ;fr Nibble-Wahl
 ldi wh,lcddatmsk           ;Referenz fr Nibblewahl
 push wl                    ;Datenbyte sichern
 sbrc wh,0                  ;wenn LCD am unteren Nibble angeschlossen ist
 swap wl                    ;dann oberes Nibble nach unten
 sbi rsport,rs              ;RS setzen
 rcall lcd_nibble           ;erstes Nibble ausgeben...
 pop wl                     ;Kopie vom Datenbyte holen
 sbrc wh,4                  ;wenn LCD am oberen Nibble angeschlossen ist
 swap wl                    ;dann unteres Nibble nach oben
 pop wh                     ;wieder frei
 sbi rsport,rs              ;RS setzen
lcd_nibble:
 andi wl,lcddatmsk          ;Daten auf vom LCD genutztes Nibble maskieren
 push wh                    ;benutztes Register sichern
 in wh,lcdport              ;ganzen Port einlesen
 cbr wh,lcddatmsk           ;nur LCD-Datenbits lschen
 or wh,wl                   ;neue Datenbits einfgen
 out lcdport,wh             ;ausgeben
 pop wh                     ;alten Wert in Register wiederherstellen
lcd_impuls:                 ;erzeugt den Enable-Puls
 sbrc flags,lcdcontroller1  ;Controller 1?
 sbi enable1port,enable1    ;ja, Enable high
 sbrc flags,lcdcontroller2  ;Controller 2?
 sbi enable2port,enable2    ;ja, Enable2 high
 rcall lcd_impulsR          ;7 Takte warten
 in wl,lcdport-2            ;im Readmode LCD auslesen
 cbi enable1port,enable1    ;Enable wieder low
 cbi enable2port,enable2    ;Enable wieder low
lcd_impulsR: 
 ret                        ;und wieder zurck...


;============================================================================
; Interne Routine, wartet bis LCD bereit ist
;----------------------------------------------------------------------------
lcd_busy:               ;wartet bis LCD bereit ist
 push wl                    ;benutzte Register
 push wh                    ;sichern
 ldi wh,0                   ;maximale Wartezeit fr Not-Abbruch
lcd_busy1:
 dec wh                     ;Wartezeit runter
 breq lcd_busy2             ;abgelaufen? Not-Abbruch...
 in wl,lcdport-1            ;nein, Datenrichtung LCD-Datenport holen 
 cbr wl,lcddatmsk           ;LCD-Datenport auf Lesezugriff
 out lcdport-1,wl           ;schalten
 sbi rwport,rw              ;R/W am LCD setzen
 cbi rsport,rs              ;RS am LCD lschen um Busy abzufragen
 rcall lcd_impuls           ;Enable-Impuls und LCD H-Nibble lesen
 swap wl                    ;Lesenibble (Bit 4...7 des LCD) nach oben
 push wl                    ;und sichern
 rcall lcd_impuls           ;Enable-Impuls und LCD L-Nibble lesen (Dummy)
 pop wl                     ;oberes Nibble wiederholen, unteres verwerfen
 rol wl                     ;Busy-Flag ins Carry
 brcs lcd_busy1             ;noch gesetzt? ja, nochmal auslesen...
lcd_busy2: 
 cbi rwport,rw              ;nein, R/W am LCD wieder lschen
 in wl,lcdport-1            ;Datenrichtung LCD-Datenport holen 
 sbr wl,lcddatmsk           ;LCD-Datenport auf Schreibzugriff
 out lcdport-1,wl           ;schalten
 pop wh                     ;Register 
 pop wl                     ;wiederherstellen
 ret


;============================================================================
; Interne Routine, erzeugt (vernichtet??) Wartezeit
;----------------------------------------------------------------------------
delay5ms:               ;5ms Pause fr manche Befehle
 push zl                    ;alten Inhalt
 push zh                    ;sichern
 ldi zh,high(clock/11000*50);Verzgerungswert fr 50ms
 ldi zl,low(clock/11000*50) ;setzen
delay5ms1:
 rcall delay5ms2            ;7 Takte vertrdeln
 sbiw zh:zl,1               ;-1 (2 weitere Takte)
 brne delay5ms1             ;0? nein, nochmal (2 Takte)
 pop zh                     ;alten Inhalt
 pop zl                     ;zurck holen
delay5ms2: 
 ret                        ;zurck...

