;Erweiterung der Ausgabe-Funktionalitt fr LCD fr AVR-Assembler (AT-Mega8).

;Die Datenausgaberoutine des LCD-Treibers sollte "lcd_data" heien.

;Es werden folgende Funktionen bereitgestellt,
;die per Macro aufgerufen werden:
;-----------------------------------------------------------------------------
;  print konst8         ;Gibt 1 Zeichen auf dem LCD aus. Das Zeichen wird als
                        ;Konstante (Byte) angegeben.
;-----------------------------------------------------------------------------
;  printhex Reg         ;Gibt den Inhalt des Registers als zweistellige
                        ;Hex-Zahl aus. Enthlt das Register eine gepackte
                        ;BCD-Zahl, so werden die beiden Ziffern ausgegeben.
                        ;Somit erbrigt sich eine separate Routine zur
                        ;Ausgabe von Packed-BCD.
;-----------------------------------------------------------------------------
;  printbin Reg         ;Gibt den Inhalt des Registers als achtstelligen
                        ;Binr-Zahl aus.
;-----------------------------------------------------------------------------
;  print8 Reg           ;Gibt den Inhalt von Reg. zweistellig aus,
                        ;wenn er kleiner als 100 ist. (Uhr)
                        ;Ab 100 erfolgt die Ausgabe dreistellig.
;-----------------------------------------------------------------------------
;  print8s Adr          ;Gibt den Inhalt von Adr (SRAM) zweistellig aus,
                        ;wenn er kleiner als 100 ist. (Uhr)
                        ;Ab 100 erfolgt die Ausgabe dreistellig.
;-----------------------------------------------------------------------------
;  print16 H-reg,L-Reg,Komma,[using]+[sig]+[fnul]
                        ;Gibt 16-Bit-Zahl als Ziffern aus
                        ;H-Reg,L-Reg enthlt die Zahl
                        ;Komma ist die Anzahl der Nachkommastellen
                        ;using erzwingt formatierte Ausgabe (Print Using)
                        ;sig erzwingt Vorzeichen
                        ;fnul erzwingt Fhrungsnullen
;-----------------------------------------------------------------------------
;  print16s Adr,Komma,[using]+[sig]+[fnul]
                        ;Gibt 16-Bit-Zahl in SRAM als Ziffern aus
                        ;Adr und ADR+1 (SRAM) enthlt die Zahl (L:H)
                        ;Komma ist die Anzahl der Nachkommastellen
                        ;using erzwingt formatierte Ausgabe (Print Using)
                        ;sig erzwingt Vorzeichen
                        ;fnul erzwingt Fhrungsnullen
;-----------------------------------------------------------------------------
;  print24 H-reg,M-Reg,L-Reg,Komma,[using]+[sig]+[fnul]
                        ;Gibt 24-Bit-Zahl als Ziffern aus
                        ;H-Reg,M-Reg,L-Reg enthlt die Zahl
                        ;Komma ist die Anzahl der Nachkommastellen
                        ;using erzwingt formatierte Ausgabe (Print Using)
                        ;sig erzwingt Vorzeichen
                        ;fnul erzwingt Fhrungsnullen
;-----------------------------------------------------------------------------
;  print24s Adr,Komma,[using]+[sig]+[fnul]
                        ;Gibt 24-Bit-Zahl aus SRAM als Ziffern aus
                        ;Adr, Adr+1 und Adr+2 (SRAM) enthlt die Zahl (L:M:H)
                        ;Komma ist die Anzahl der Nachkommastellen
                        ;using erzwingt formatierte Ausgabe (Print Using)
                        ;sig erzwingt Vorzeichen
                        ;fnul erzwingt Fhrungsnullen
;-----------------------------------------------------------------------------
;  printui H-Reg,L-Reg  ;Gibt den Inhalt der Zahl in H-reg:L-Reg 5-stellig aus
                        ;Ist inzwischen dank "print16" berflssig
;-----------------------------------------------------------------------------
;  printt adr16,Reg     ;Gibt einen Text aus, dessen Adresse in einer
                        ;Indextabelle steht.
                        ;adr16 ist die Basisadresse der Indextabelle,
                        ;Reg ist ein Register, das den den Eintrag in
                        ;der Tabelle auswhlt.
                        ;Dient zum Ausgeben von indizierten Texten wie
                        ;Menpunkte, Wochentage usw.
;-----------------------------------------------------------------------------
;  printts adr16,Adr    ;Gibt einen Text aus, dessen Adresse in einer
                        ;Indextabelle steht.
                        ;adr16 ist die Basisadresse der Indextabelle,
                        ;Adr ist eine SRAM-Zelle, die den den Eintrag in
                        ;der Tabelle auswhlt.
                        ;Dient zum Ausgeben von indizierten Texten wie
                        ;Menpunkte, Wochentage usw.
;-----------------------------------------------------------------------------
;  printf adr16         ;Gibt den 0-terminierten Text an Adr.16 im Flash aus
;-----------------------------------------------------------------------------
;  prints adr16         ;Gibt den 0-terminierten Text an Adr.16 im SRAM aus
;-----------------------------------------------------------------------------
;  printe adr16         ;Gibt den 0-terminierten Text an Adr.16 im EEPROM aus
;=============================================================================
;Alle benutzten Register werden whrend der Abarbeitung gesichert.
;Es werden neben den Indexregistern zwei weitere Register 
;"wl" (r24) und "wh" (r25) des Hauptprogramms genutzt (working low und
;working high). Sie mssen dort deklariert werden. Es wird davon ausgegangen,
;dass die Routine "lcd_data" (Bestandteil der LCD-Treiber-Routinensammlung)
;ein Zeichen auf das LCD ausgibt. Die Positionierung wird mit dem Macro
;"locate" erreicht, das in einer separaten Include-Datei vorliegt.


.equ using=0b00100000   ;Flag fr formatierte Ausgabe (Print Using)
.equ sig = 0b01000000   ;Flag fr Vorzeichen-Zahl (signed Integer)
.equ fnul= 0b10000000   ;Flag fr Fhrungsnullen


.macro print ;Konstante als Byte
                        ;Gibt ein Zeichen (Konstante) an LCD aus
 push wl
 ldi wl,@0                  ;Das Zeichen als Konstante
 rcall lcd_data             ;an LCD ausgeben
 pop wl
.endmacro

.macro print8 ;Registername
                        ;Gibt Zahl im Register 2- oder 3-stellig am LCD aus.
                        ;Parameter bestimmt das Register, in dem die Zahl liegt.
                        ;Dieses Register wird nicht verndert.
                        ;Kopie der Zahl temporr in xl,
 push xl                    ;verwendete Register sichern
 mov xl,@0                  ;Kopie vom Quellregister
 rcall lcd_print8           ;Aufruf...
 pop xl                     ;verwendete Register wiederherstellen
.endmacro               ;

.macro print16 ;H-Reg,L-Reg,Kommastellen,using+sign+fnul
 push xh                    ;verwendete Register
 push xl                    ;sichern
 push wh
 push wl
 mov xl,@1                  ;Low-Byte
 mov xh,@0                  ;High-Byte
 ldi wh,(@3 & $e0)|16|(6-(@2 & 15))   ;Mode-Flags und Komma-Position
 rcall lcd_print16          ;Aufruf
 pop wl
 pop wh
 pop xl                     ;verwendete Register
 pop xh                     ;wiederherstellen
.endmacro

.macro print24 ;H-Reg,M-Reg,L-Reg,Kommastellen,using+sign+fnul
 push yl
 push xh                    ;verwendete Register
 push xl                    ;sichern
 push wh
 push wl
 mov xl,@2                  ;Low-Byte
 mov xh,@1                  ;Byte2
 mov yl,@0                  ;Byte3
 ldi wh,(@4 & $e0)|16|(9-(@3 & 15))   ;Mode-Flags und Komma-Position
 rcall lcd_print24          ;Aufruf
 pop wl
 pop wh
 pop xl                     ;verwendete Register
 pop xh                     ;wiederherstellen
 pop yl
.endmacro

.macro print8s ;Adresse im SRAM
                        ;Gibt Zahl im Register 2- oder 3-stellig am LCD aus.
                        ;Parameter bestimmt die Adresse im SRAM,
                        ;in dem die Zahl liegt.
                        ;Dieser Inhalt wird nicht verndert.
                        ;Kopie der Zahl temporr in xl,
 push xl                    ;verwendete Register sichern
 lds xl,@0                  ;Kopie vom Quellregister
 rcall lcd_print8           ;Aufruf...
 pop xl                     ;verwendete Register wiederherstellen
.endmacro               ;

.macro print16s ;Adresse,Kommastellen,using+sign+fnul
 push xh                    ;verwendete Register
 push xl                    ;sichern
 push wh
 push wl
 lds xl,@0                  ;Low-Byte aus SRAM
 lds xh,@0+1                ;High-Byte aus SRAM
 ldi wh,(@2 & $e0)|16|(6-(@1 & 15))   ;Mode-Flags und Komma-Position
 rcall lcd_print16          ;Aufruf
 pop wl
 pop wh
 pop xl                     ;verwendete Register
 pop xh                     ;wiederherstellen
.endmacro

.macro print24s ;Adresse,Kommastellen,using+sign+fnul
 push yl
 push xh                    ;verwendete Register
 push xl                    ;sichern
 push wh
 push wl
 lds yl,@0                  ;Byte3
 lds xh,@0+1                ;Byte2
 lds xl,@0+2                ;Low-Byte
 ldi wh,(@2 & $e0)|16|(9-(@1 & 15))   ;Mode-Flags und Komma-Position
 rcall lcd_print24          ;Aufruf
 pop wl
 pop wh
 pop xl                     ;verwendete Register
 pop xh                     ;wiederherstellen
 pop yl
.endmacro

.macro printui ;H-Register, L-Register
                        ;Gibt u-Integer im Registerpaar H:L 5-stellig an LCD aus.
                        ;Die Parameter geben die Register H und L an.
                        ;Diese Register werden nicht verndert.
                        ;Kopie der Zahl in XH und XL
 push xh                    ;verwendete Register
 push xl                    ;sichern
 push wh
 push wl
 mov xl,@1                  ;L-Byte kopieren
 mov xh,@0                  ;H-Byte kopieren
 ldi wh,15                  ;kein Dezimalpunkt, keine Nullunterdrckung
 rcall lcd_printui          ;Funkion aufrufen...
 pop wl
 pop wh
 pop xl                     ;verwendete Register
 pop xh                     ;wiederherstellen
.endmacro

.macro printt ;Adresse der Tabelle, Index auf Tabelle
 push zh                    ;verwendete Register
 push zl                    ;sichern
 push wl
 ldi zh,high(@0*2)          ;Basisadresse
 ldi zl,low(@0*2)           ;der Indextabelle
 mov wl,@1                  ;Index holen
 rcall lcd_printt           ;Aufruf
 pop wl                     ;verwendete
 pop zl                     ;Register
 pop zh                     ;wiederherstellen
.endmacro

.macro printts ;Adresse der Tabelle, Index auf Tabelle (SRAM)
 push zh                    ;verwendete Register
 push zl                    ;sichern
 push wl
 ldi zh,high(@0*2)          ;Basisadresse
 ldi zl,low(@0*2)           ;der Indextabelle
 lds wl,@1                  ;Index aus SRAM holen
 rcall lcd_printt           ;Aufruf
 pop wl                     ;verwendete
 pop zl                     ;Register
 pop zh                     ;wiederherstellen
.endmacro

.macro printf ;Startadresse des Strings im Flash
                        ;Gibt einen String aus dem Flash an LCD aus
                        ;Ein Parameter beschreibt die Startadresse, das Ende ist $00
 push zh                    ;verwendete Register
 push zl                    ;sichern
 ldi zh,high(2*@0)          ;Pointer
 ldi zl,low(2*@0)           ;setzen
 rcall lcd_printf           ;Aufruf...
 pop zl                     ;verwendete Register
 pop zh                     ;wiederherstellen
.endmacro

.macro prints ;Startadresse des Strings im SRAM
                        ;Gibt einen String aus dem SRAM an LCD aus
                        ;Ein Parameter beschreibt die Startadresse, das Ende ist $00
 push zh                    ;verwendete Register
 push zl                    ;sichern
 ldi zh,high(@0)            ;Pointer
 ldi zl,low(@0)             ;setzen
 rcall lcd_prints           ;Aufruf...
 pop zl                     ;verwendete Register
 pop zh                     ;wiederherstellen
.endmacro

.macro printe ;Startadresse des Strings im EEPROM
                        ;Gibt einen String aus dem EEPROM an LCD aus
                        ;Ein Parameter beschreibt die Startadresse, das Ende ist $00
 push zh                    ;verwendete Register
 push zl                    ;sichern
 ldi zh,high(@0)            ;Pointer
 ldi zl,low(@0)             ;setzen
 rcall lcd_printe           ;Aufruf...
 pop zl                     ;verwendete Register
 pop zh                     ;wiederherstellen
.endmacro

.macro printhex ;Register
 push wl                    ;Reg sichern
 mov wl,@0                  ;Daten bernehmen
 rcall lcd_printhex         ;Aufruf...
 pop wl                     ;Reg wiederherstellen
.endmacro

.macro printbin ;Register
 push wh                    ;Reg sichern
 mov wh,@0                  ;Daten bernehmen
 rcall lcd_printbin         ;Aufruf...
 pop wh                     ;Reg wiederherstellen
.endmacro

;---------------------------

lcd_print8:             ;Wird vom Makro aufgerufen. Gibt Byte als 2 oder 3 Ziffern an LCD aus.
 push wl                    ;Reg sichern
 ldi wl,-1+'0'              ;Hunderter-Stelle als ASCII-Zeichen, Zahl ist positiv
 inc wl                     ;Hunderter hoch und
 subi xl,100                ;100 subtrahieren bis negativ wird
 brsh pc-2                  ;negativ? nein, 2 Zeilen hoch
 cpi wl,'0'                 ;ja, ist Ziffer = "0"?
 breq pc+2                  ;ja, nicht ausgeben...
 rcall lcd_data             ;Hunderter ausgeben...
 ldi wl,10+'0'              ;Zehner-Stelle als ASCII-Zeichen, Zahl ist negativ
 dec wl                     ;Zehner runter und
 subi xl,-10                ;10 addieren bis positiv wird
 brlo pc-2                  ;positiv? nein, 2 Zeilen hoch...
 rcall lcd_data             ;ja, Zehner Stelle ausgeben...
 ldi wl,'0'                 ;ASCII-0
 add wl,xl                  ;Einer addieren (Rest war ja positiv)
 rcall lcd_data             ;Einer ausgeben...
 pop wl                     ;Reg wiederherstellen
 ret                        ;zurck

lcd_print24:            ;Wird von Macro aufgerufen. Gibt Word als Zahl in mehreren Modi aus
 sbrs wh,6                  ;Signum-Flag gesetzt? ja...
 rjmp lcd_print24u          ;nein, als unsigned behandeln...
 sbrs yl,7                  ;Wert negativ? ja...
 rjmp lcd_print24plus       ;nein, positiv...
 ldi wl,'-'                 ;Vorzeichen
 rcall lcd_data             ;ausgeben
 neg xl                     ;Zahl positiv
 com xh                     ;machen
 rjmp lcd_print24u          ;weiter...
lcd_print24plus:        ;Pluszeichen ausgeben
 ldi wl,'+'                 ;Pluszeichen
 rcall lcd_data             ;an LCD
lcd_print24u:           ;24-Bit-Zahl zerlegen
 ldi wl,-1+'0'              ;Ziffer fr ZehnMillon
 inc wl                     ;ZehnMio Stelle hoch und
 subi xl,low(10000000)      ;von Zahl 10 Mio subtrahieren
 sbci xh,high(10000000)     ;bis Zahl negativ wird
 sbci yl,byte3(10000000)    ;
 brcc pc-4                  ;schon negativ? nein...
 rcall lcd_format           ;ja, Ziffer ausgeben (Zehn-Mio)
 ldi wl,10+'0'              ;Ziffer fr Mollion, Zahl ist jetzt negativ
 dec wl                     ;Mio Stelle verkleinern und
 subi xl,low(-1000000)      ;1 Mio zur (negativen) Zahl addieren,
 sbci xh,high(-1000000)     ;bis Zahl wieder positiv wird
 sbci yl,byte3(-1000000)    ;
 brcs pc-4                  ;noch negativ? ja...
 rcall lcd_format           ;nein, Ziffer ausgeben (1 Mio)
 ldi wl,-1+'0'              ;Ziffer fr Hunderttausender
 inc wl                     ;Hunderttausender Stelle hoch und
 subi xl,low(100000)        ;von Zahl 100000 subtrahieren
 sbci xh,high(100000)       ;bis Zahl negativ wird
 sbci yl,byte3(100000)      ;
 brcc pc-4                  ;schon negativ? nein...
 rcall lcd_format           ;ja, Ziffer ausgeben (Hunderttausender)
 ldi wl,10+'0'              ;Ziffer fr Tausender, Zahl ist jetzt negativ
 dec wl                     ;Tausender Stelle verkleinern und
 subi xl,low(-10000)        ;1 Mio zur (negativen) Zahl addieren,
 sbci xh,high(-10000)       ;bis Zahl wieder positiv wird
 sbci yl,byte3(-10000)      ;
 brcs pc-4                  ;noch negativ? ja...
 subi xl,low(10000)         ;nein, 10000 von der (positiven) Zahl subtrahieren,
 sbci xh,high(10000)        ;Zahl wieder negativ wird und ins Raster der
 rjmp lcd_print16_a         ;16-Bit-Routine passt, die den Rest zerlegt und ausgibt...


lcd_print16:            ;Wird von Macro aufgerufen. Gibt Word als Zahl in mehreren Modi aus
 sbrs wh,6                  ;Signum-Flag gesetzt? ja...
 rjmp lcd_printui           ;nein, als unsigned-Int behandeln...
 sbrs xh,7                  ;Wert negativ? ja...
 rjmp lcd_printsig          ;nein, positiv...
 ldi wl,'-'                 ;Vorzeichen
 rcall lcd_data             ;ausgeben
 neg xl                     ;Zahl positiv
 com xh                     ;machen
lcd_printui:            ;Wird von Makro aufgerufen. Gibt unsigned-Integer als 5 Ziffern an LCD aus
 ldi wl,-1+'0'              ;Ziffer fr Zehntausender
 inc wl                     ;Zehntausender Stelle hoch und
 subi xl,low(10000)         ;von Zahl 10000 subtrahieren
 sbci xh,high(10000)        ;bis Zahl negativ wird
 brcc pc-3                  ;schon negativ? nein...
lcd_print16_a:          ;Einsprung aus 24-Bit-Routine
 rcall lcd_format           ;ja, Ziffer ausgeben (Zehntausender)
 ldi wl,10+'0'              ;Ziffer fr Tausender, Zahl ist jetzt negativ
 dec wl                     ;Tausender Stelle verkleinern und
 subi xl,low(-1000)         ;1000 zur (negativen) Zahl addieren,
 sbci xh,high(-1000)        ;bis Zahl wieder positiv wird
 brcs pc-3                  ;noch negativ? ja...
 rcall lcd_format           ;nein, Ziffer ausgeben (Tausender)
 ldi wl,-1+'0'              ;Ziffer fr Hunderter (Zahl ist jetzt positiv)
 inc wl                     ;Hunderter Stelle hoch und
 subi xl,low(100)           ;von Zahl 100 subtrahieren
 sbci xh,high(100)          ;bis Zahl negativ wird
 brcc pc-3                  ;schon negativ? nein...
 rcall lcd_format           ;ja, Ziffer ausgeben (Hunderter)
 ldi wl,10+'0'              ;Zehner-Stelle als ASCII-Zeichen, Zahl ist negativ
 dec wl                     ;Zehner runter und
 subi xl,-10                ;10 addieren bis positiv wird
 brlo pc-2                  ;positiv? nein, 2 Zeilen hoch...
 rcall lcd_format           ;ja, Zehner Stelle ausgeben...
 ldi wl,'0'                 ;ASCII-0
 add wl,xl                  ;Einer addieren (Rest war ja positiv)
 rcall lcd_format           ;Einer ausgeben
 andi wh,using              ;Fhrungsnullen oder Using aktiv?
 brne pc+3                  ;ja, also konstante Textlnge...
 ldi wl,' '                 ;nein, variable Textlnge, deshalb
 rjmp lcd_data              ;Leerzeichen anhngen um alte Ziffer zu lschen
 ret                        ;zurck
lcd_printsig:
 ldi wl,'+'                 ;Vorzeichen
 rcall lcd_data             ;ausgeben
 rjmp lcd_printui           ;Zahl ausgeben
lcd_format:
 dec wh                     ;Kommazhler runter
 push wh                    ;Mode-Flags sichern
 andi wh,$0f                ;Kommazhler isolieren
 cpi wh,1                   ;Letzte Stelle vor dem Komma?
 pop wh                     ;(Mode-Flags wegen Vernderung holen)
 brne pc+2                  ;nein...
 sbr wh,fnul                ;Nullausgabe aktivieren
 push wh                    ;genderte Mode-Flags nochmal sichern
 andi wh,$0f                ;Kommazhler isolieren (wegen Zero-Flag)
 pop wh                     ;Mode-Flags wiederherstellen
 brne lcd_keinkomma         ;Kommazhler nicht Null...
 push wl                    ;Ziffer sichern
 ldi wl,','                 ;Komma
 rcall lcd_data             ;ausgeben
 pop wl                     ;Ziffer wiederherstellen
lcd_keinkomma:
 cpi wl,'0'                 ;Ziffer = "0"?
 breq lcd_format0           ;ja...
 sbr wh,fnul                ;nein, knftige Nullen nicht mehr unterdrcken
 rjmp lcd_data              ;Ziffer ausgeben und zurck...
lcd_format0:
 sbrs wh,7                  ;Nullen ausgeben? ja...
 ldi wl,' '                 ;nein, Null durch Leerzeichen ersetzen
 sbrc wh,7                  ;Nullen ausgeben? nein...
 rjmp lcd_data              ;ja, Ziffer "0" ausgeben und zurck...
 sbrc wh,5                  ;Print Using? nein...
 rjmp lcd_data              ;ja, Leerzeichen ausgeben und zurck...
 ret                        ;nein, ohne Ausgabe zurck...

lcd_printt:             ;Wird von Makro aufgerufen. Gibt indizierten Text aus
 push wh
 lsl wl                     ;Index mal 2 (2 Bytes pro Eintrag)
 add zl,wl                  ;Index addieren
 adc zh,null                ;bertrag auch
 lpm wl,z+                  ;Low-Byte holen
 lpm wh,z+                  ;High-Byte holen
 mov zl,wl                  ;in Z-Pointer
 mov zh,wh                  ;kopieren
 lsl zl                     ;Index auf Text
 rol zh                     ;mal 2 (Byteweise Adressierung)
 rcall lcd_printf           ;Text ausgeben...
 pop wh
 ret

lcd_printf:             ;Wird vom Makro aufgerufen. Gibt Flash-String an LCD aus.
 push wl
 lpm wl,z+                  ;Zeichen holen
 tst wl                     ;Ende-Kennung? 
 breq pc+3                  ;ja...
 rcall lcd_data             ;nein, ausgeben
 rjmp pc-4                  ;nochmal
 pop wl
 ret

lcd_prints:             ;Wird vom Makro aufgerufen. Gibt SRAM-String an LCD aus.
 push wl
 ld wl,z+                   ;Zeichen holen
 tst wl                     ;Ende-Kennung? 
 breq pc+3                  ;ja...
 rcall lcd_data             ;nein, ausgeben
 rjmp pc-4                  ;nochmal
 pop wl
 ret

lcd_printe:             ;Wird vom Makro aufgerufen. Gibt EEPROM-String an LCD aus.
 push wl
 out eearh,zh               ;Adresse
 out eearl,zl               ;setzen
 adiw zh:zl,1               ;und fr nchste Runde erhhen
 sbi eecr,eere              ;EEP lesen
 in wl,eedr                 ;Zeichen holen
 tst wl                     ;Ende-Kennung? 
 breq pc+3                  ;ja...
 rcall lcd_data             ;nein, ausgeben
 rjmp pc-8                  ;nochmal
 pop wl
 ret

lcd_printhex:           ;Wird von Makro aufgerufen. Gibt Byte als HEX aus.
 push wl                    ;fr zweite Ziffer sichern
 swap wl                    ;oberes Nibble zuerst
 rcall lcd_printhex1        ;ausgeben...
 pop wl                     ;unteres Nibble
lcd_printhex1:          ;Nibble ausgeben
 andi wl,15                 ;nur unteres Nibble interessiert uns
 ori wl,'0'                 ;zur Ascii-Ziffer wandeln
 cpi wl,'9'+1               ;Wert ber 9?
 brmi lcd_printhex2         ;nein, so lassen...
 subi wl,-39                ;zum Kleinbuchstaben wandeln
; subi wl,-7                 ;zum Grobuchstaben wandeln
lcd_printhex2:
 rjmp lcd_data              ;Zeichen ausgeben und zurck...

lcd_printbin:   ;Wird von Makro aufgerufen. Gibt Byte als 8 BIN-Ziffern aus.
 push wl                    ;Reg sichern
 sec                        ;gesetztes Carry als Ende-Indikator ins Register,
 rol wh                     ;oberstes Bit ins Carry schieben
lcd_printbin1:
 ldi wl,'0'                 ;erstmal Ziffer "0"
 adc wl,null                ;Ziffer "1", falls Carry gesetzt war
 rcall lcd_data             ;Ziffer ausgeben
 lsl wh                     ;nchstes Bit ins Carry schieben
 brne lcd_printbin1         ;Indikator-Bit noch im Register? ja...
 pop wl                     ;nein, im Carry, wird nicht mit ausgegeben,
 ret                        ;Register wiederherstellen und zurck...


