;Baustelle...

;Digitaluhr mit AT-Mega8 und LCD
;nach einem Hilferuf von avusgalaxy

.nolist                     ;List der Include-Datei unterdrcken
.include "m8def.inc"        ;Hardwaredefinitionen einbinden
.list                       ;Programm listen
.listmac                    ;Makros listen
.equ clock=8000000          ;Taktfrequenz des AVR

.equ tim1zu=10000           ;Timer1-Interrupt-Takt
.equ teilfaktor=100         ;Zhlumfang des Teilers
.equ tap=pinb               ;Port fr die Tasten
    .equ tli=1                  ;Taste links
    .equ tre=3                  ;Taste rechts
    .equ tho=4                  ;Taste hoch
    .equ tru=5                  ;Taste runter
.equ alltast=(1<<tru)|(1<<tre)|(1<<tho)|(1<<tli) ;Maske aller Tasten
.equ t0port=portd               ;Port T0
.equ t0pin=4                    ;Pin T0
.equ icpport=portd              ;ICP-Port
.equ icppin=0                   ;ICP-Pin
.equ anaport=portc              ;Port Analogeingang
.equ anapin=4                   ;Pin Analogeingang
.equ ledpin=5                   ;Pin LED zur Akkuspannungsmessung


.equ lcdstromport=portd     ;Port Stromversorgung LCD
.equ lcdstrompin=1          ;Pin Stromversorgung LCD


.dseg                   ;SRAM-Variablen
sekunde:        .byte 1     ;aktuelle Sekunde
minute:         .byte 1     ;aktuelle Minute
stunde:         .byte 1     ;aktuelle Stunde
wotag:          .byte 1     ;aktueller Wochentag
frequenz:       .byte 3     ;gemessene Frequenz (L-Byte zuerst)
impulsl:        .byte 3     ;gemessene Impulspausendauer (L zuerst)
impulsh:        .byte 3     ;gemessene Impulsdauer (L zuerst)
batt0:          .byte 1     ;Batteriezustand, Nachkommastellen (Mittelwert)
batt1:          .byte 1     ;Batteriezustand, volle Prozentwerte

astring:        .byte 27    ;Startadresse fr LCD-Ausgabestring im S-RAM


.def null=r2                ;immer Null
.def srsk=r3                ;SREG-Siko
.def tim1u=r3               ;bertrag vom Timer1
.def icpl=r4                ;letzter ICP
.def icph=r5                ;Wert
.def icpu=r6                ;mit bertrag
.def tim0u1=r7              ;erster bertrag vom Timer0 (zweites Byte)
.def tim0u2=r8              ;zweiter bertrag vom Timer0 (drittes Byte)

.def tz0=r11                ;Tasten-Prellzhler Bit0
.def tz1=r12                ;Tasten-Prellzhler Bit1
.def tas=r13                ;Tastenstatus (gltig)

.def mp=r19                 ;Menpunkt
.def teiler=r20             ;Teiler fr Timer (1 Sekunde)
.def posi=r21
.def tfl=r22                ;Tasten-Flags (melden Tastendruck)
.def flags=r23              ;diverse Flags
    .equ neusek=0               ;neue Sekunde
    .equ neuhalb=1              ;neue halbe Sekunde
    .equ hundertstel=2          ;neue Hundertstel Sekunde
    .equ uhraktiv=3             ;Flag fr Uhrzeitausgabe
    .equ uhrhalt=4              ;Flag fr angehaltene Uhr
    .equ uhrpiep=5              ;Flag fr aktivierten Piepser
    .equ lcdcontroller1=6       ;Flag fr Controller 1
    .equ lcdcontroller2=7       ;Flag fr Controller 2
.def wl=r24                 ;Working L
.def wh=r25                 ;Working H

.cseg
.org 0              ;Reset- und Interrupt-Vektoren AT-Mega 8
 rjmp reset         ;Reset Handler
 reti; rjmp EXT_INT0      ;IRQ0 Handler
 reti; rjmp EXT_INT1      ;IRQ1 Handler
 reti; rjmp TIM2_COMP     ;Timer2 Compare Handler
 reti; rjmp TIM2_OVF      ;Timer2 Overflow Handler
 rjmp TIM1_CAPT     ;Timer1 Capture Handler
 rjmp TIM1_COMPA    ;Timer1 CompareA Handler
 reti; rjmp TIM1_COMPB    ;Timer1 CompareB Handler
 rjmp TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 reti; rjmp SPI_STC       ;SPI Transfer Complete Handler
 reti; rjmp USART_RXC     ;USART RX Complete Handler
 reti; rjmp USART_UDRE    ;UDR Empty Handler
 reti; rjmp USART_TXC     ;USART TX Complete Handler
 reti; rjmp ADC           ;ADC Conversion Complete Handler
 reti; rjmp EE_RDY        ;EEPROM Ready Handler
 reti; rjmp ANA_COMP      ;Analog Comparator Handler
 reti; rjmp TWSI          ;Two-wire Serial Interface Handler
 reti; rjmp SPM_RDY       ;Store Program Memory Ready Handler

.include"LCDprint.inc"      ;LCD-Ausgabefunktionen
.include"LCD_4x27.inc"      ;LCD-Treiber-Routinen fr LCDs mit M50530



reset:
 ldi wl,low(ramend)         ;Stackpointer initialisieren
 out SPL,wl
 ldi wl,high(ramend)
 out SPH,wl
 clr null                   ;immer Null
 clr tas
 clr tz0
 clr tz1

 ldi wl,1<<se               ;Sleep-Mode Idle aktivieren
 out mcucr,wl
 
 ldi wl,alltast             ;Pull-Ups
 out tap+2,wl               ;fr Tasten einschalten

 ldi flags,1<<uhraktiv      ;Uhrzeitanzeige einschalten
 ldi mp,32
 rcall lcd_stromein         ;LCD einschalten und initialisieren
 locate 0,3                 ;Mentext
 printf mt_uhr              ;ausgeben 

 in wl,mcucsr               ;Reset-Art ermitteln
 sbrc wl,extrf              ;Pin-Reset? nein...
 rjmp reset1                ;ja, Uhr nicht stellen...
 clr wl                     ;Voreinstellung
 sts wotag,wl               ;Wochentag
 ldi wl,12                  ;
 sts stunde,wl              ;Stunde
 ldi wl,0
 sts minute,wl              ;Minute
 ldi wl,0
 sts sekunde,wl             ;Sekunde
 ldi mp,67                  ;Uhr stellen
 sbr tfl,1<<tre             ;Tastendruck vortuschen, um Men aufzurufen
 locate 0,3                 ;Mentext
 printf mt_67               ;ausgeben 
reset1:


 ldi wl,1<<acd              ;Analog-Comparator aus
 out acsr,wl                ;Strom sparen

 sbi t0port,t0pin           ;Pull-Up fr T0 ein
 sbi icpport,icppin         ;Pull-Up fr ICP ein

 ldi wl,7                   ;Timer0 mit externem Takt an T0 (PD4)
 out tccr0,wl               ;starten
 ldi wl,(1<<icnc1)|(1<<cs11);ICP-Rauschunterdrckung ein und
 out tccr1b,wl              ;Timer1 mit Vorteiler 8:1 starten
 ldi wl,low(tim1zu)         ;Interrupt-
 ldi wh,high(tim1zu)        ;Intervall
 out ocr1ah,wh              ;Timer1
 out ocr1al,wl              ;setzen
 ldi wl,(1<<ocie1a)|(1<<tov1)|(1<<ticie1)|(1<<toie0)
; ldi wl,(1<<ocie1a)|(1<<tov1)

 out TIMSK,wl               ;Int ein: Timer1-Compare A, ICP, OVF 0 und 1

 sei                        ;Interrupts freigeben

 ldi wl,50                  ;Startwert fr Mittelwert
 sts batt1,wl               ;Akkuzustand ins SRAM
 sts batt0,null             ;Nachkommastellen lschen
 ldi wl,(1<<adlar)+14       ;linksbndig, Bandgap an
 out admux,wl               ;ADC legen
 ldi wl,(1<<aden)|(1<<adsc)|(1<<adfr)+6 ;ADC freilaufend mit Vorteiler 64
 out adcsra,wl              ;einschalten


tastaus:                ;Tasten puffer leeren und mainloop
 clr tfl                    ;Tastenflags lschen
 sbis lcdstromport,lcdstrompin ;Display aktiv? ja...
 rjmp mainloop              ;nein, Ausgabe unterdrcken...

 locate 0,24         ;Debug, Position
 print8 mp          ;Debug, Menpunkt

mainloop:               ;Hauptschleife
 sbrc flags,neusek          ;neue Sekunde da?? nein...
 rjmp neusekunde            ;ja...
 sbrc flags,neuhalb         ;neue halbe Sekunde da?? nein...
 rjmp halbsekunde           ;ja...
 ;Hier ist Platz fr weitere Main-Jobs
 ;Das geht auch mit rcall, mit rjmp kann man aber eine
 ;Hirarchie nach Wichtigkeit realisieren (die Wichtigeren nach oben)
 andi tfl,alltast           ;nur benutzte Tasten prfen
 breq mainloop1             ;keine Taste gedrckt...
 rjmp menu                  ;Tastendruck auswerten...
mainloop1:
 sbrc flags,hundertstel     ;neue hundertstel Sekunde da?? nein...
 rjmp akkutest              ;ja...
 sleep                      ;bis zum nchsten Int pennen
 rjmp mainloop


TIM1_CAPT:              ;ISR Input-Capture (Impulsmessung)
 in srsk,sreg               ;SREG sichern (Exklusivregister)
 push xh                    ;benutzte Register
 push xl                    ;sichern
 in xl,tccr1b               ;Flanke holen
 sbrc xl,ices1              ;ICP-Flanke? fallende...
 rjmp tim1_capt1            ;steigende...
;fallende Flanke an icp1
 sbr xl,1<<ices1            ;auf steigende umschalten
 out tccr1b,xl              ;und zurckschreiben
 in xl,icr1l                ;Capture-Wert
 in xh,icr1h                ;holen,
 push tim1u                 ;neue Messwerte sichern
 push xh
 push xl
 sub xl,icpl                ;Differenz bilden
 sbc xh,icph
 sbc tim1u,icpu
 sts impulsl,xl             ;Differenz ins SRAM
 sts impulsl+1,xh
 sts impulsl+2,tim1u
 pop icpl                   ;neue Werte als alte Werte merken
 pop icph
 pop icpu
 rjmp tim1_captende
tim1_capt1:             ;steigende Flanke an ICP1
 cbr xl,1<<ices1            ;auf fallende umschalten
 out tccr1b,xl              ;und zurckschreiben
 in xl,icr1l                ;Capture-Wert
 in xh,icr1h                ;holen,
 push tim1u                 ;neue Messwerte sichern
 push xh
 push xl
 sub xl,icpl                ;Differenz bilden
 sbc xh,icph
 sbc tim1u,icpu
 sts impulsh,xl             ;Differenz ins SRAM
 sts impulsh+1,xh
 sts impulsh+2,tim1u
 pop icpl                   ;neue Werte als alte Werte merken
 pop icph
 pop icpu
tim1_captende:
 pop xl                     ;benutzte Register und
 pop xh
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...


TIM1_COMPA:             ;ISR Timer1-Interrupt (alle 10ms)
 in srsk,sreg               ;SREG sichern (Exklusivregister)
 push xh                    ;benutzte Register
 push xl                    ;sichern
 in xl,ocr1al               ;Weckzeit
 in xh,ocr1ah               ;holen,
 subi xl,low(-tim1zu)       ;Intervall
 sbci xh,high(-tim1zu)      ;dazu,
 out ocr1ah,xh              ;und wieder
 out ocr1al,xl              ;in den Timer
 ;xl und xh knnen jetzt innerhalb dieser ISR frei benutzt werden 
 
Tastenabfrage:     ;Entprellroutine geklaut bei Peter Dannegger...
 in xl,tap               ;Tastenport einlesen (gedrckt=L)
 com xl                  ;invertieren (gedrckt=H)
 eor xl,tas              ;nur nderungen werden H
 and tz0,xl              ;Prellzhler unvernderter Tasten lschen (Bit0)
 and tz1,xl              ;Prellzhler unvernderter Tasten lschen (Bit1)
 com tz0                   ;L-Bit zhlen 0,2,->1, 1,3,->0
 eor tz1,tz0               ;H-Bit zhlen 0,2,->tz1 toggeln
 and xl,tz0              ;nderungen nur dann erhalten, wenn im Prellzhler
 and xl,tz1              ;beide Bits gesetzt sind (Zhlerstand 3)
 eor tas,xl              ;erhaltene nderungen toggeln alten (gltigen) Tastenstatus
 and xl,tas              ;nur (neu) gedrckte Tastenbits bleiben erhalten
 or tfl,xl               ;und zugehrige Bits setzen (gelscht wird nach Abarbeitung)
 ;in "tas" steht jetzt der gltige Tastenzustand,
 ;in "tfl" die Flags der neu gedrckten, noch nicht abgearbeiteten Tasten...
 ;xl ist jetzt wieder frei fr weitere temporre Zwecke in der ISR


 sbr flags,1<<hundertstel   ;neue hundertstel Sekunde 
 inc teiler                 ;Sekundenvorteiler hoch
 cpi teiler,teilfaktor      ;Endwert erreicht?
 brne nixvollsek            ;nein...
 sbr flags,1<<neusek        ;ja, Flag setzen,
 clr teiler                 ;und Teiler lschen
nixvollsek: 
 cpi teiler,teilfaktor/2    ;Halbe Sekunde?
 brne nixhalbsek            ;nein...
 sbr flags,1<<neuhalb       ;ja, Flag setzen
nixhalbsek:
 pop xl                     ;benutzte Register
 pop xh                     ;wiederherstellen
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...

tim1_ovf:               ;berlauf Timer1
 in srsk,sreg               ;Status sichern
 inc tim1u                  ;drittes Timerbyte erhhen
 out sreg,srsk              ;Status
 reti

tim0_ovf:               ;berlauf Timer0
 in srsk,sreg               ;Status sichern
 inc tim0u1                 ;zweites Timerbyte erhhen
 brne pc+2                  ;bertrag? nein...
 inc tim0u2                 ;drittes Timerbyte erhhen
 out sreg,srsk              ;Status
 reti

neusekunde:             ;neue Sekunde addieren (Job der Mainloop)
 cbr flags,1<<neusek        ;Flag lschen (Job ist in Arbeit...)

 sbrc flags,uhrhalt         ;Uhr angehalten? nein...
 rjmp ausgabe               ;ja, nicht hochzhlen...

 lds wh,sekunde             ;Sekunde aus SRAM holen
 inc wh                     ;erhhen
 sts sekunde,wh             ;erstmal zurck ins SRAM
 cpi wh,60                  ;berlauf?
 brlo ausgabe               ;nein...
 clr wh                     ;ja, von 0 beginnen
 sts sekunde,wh             ;Korrektur zurck ins SRAM

 lds wh,minute              ;Minute aus SRAM holen
 inc wh                     ;erhhen
 sts minute,wh              ;erstmal zurck ins SRAM
 cpi wh,60                  ;berlauf?
 brlo ausgabe               ;nein...
 clr wh                     ;ja, von 0 beginnen
 sts minute,wh              ;Korrektur zurck ins SRAM

 lds wh,stunde              ;Sekunde aus SRAM holen
 inc wh                     ;erhhen
 sts stunde,wh              ;erstmal zurck ins SRAM
 cpi wh,24                  ;berlauf?
 brlo ausgabe               ;nein...
 clr wh                     ;ja, von 0 beginnen
 sts stunde,wh              ;Korrektur zurck ins SRAM

 lds wh,wotag               ;Wochentag aus SRAM holen
 inc wh                     ;erhhen
 sts wotag,wh               ;erstmal zurck ins SRAM
 cpi wh,7                   ;berlauf?
 brlo ausgabe               ;nein...
 clr wh                     ;ja, von 0 beginnen
 sts wotag,wh               ;Korrektur zurck ins SRAM

ausgabe:                ;Wochentag und Uhrzeit ausgeben
 sbis lcdstromport,lcdstrompin;Display aktiv? ja...
 rjmp mainloop              ;nein, Ausgabe unterdrcken...
 sbrs flags,uhraktiv        ;Uhr aktiv?
 rjmp messausgabe           ;nein, Messwerte ausgeben...

; rjmp mainloop              ;Debug, vorzeitig deaktiviert

 locate 1,3                 ;Ausgabeposition Wochentagtext
 printts wochentag,wotag

 locate 1,15                ;Ausgabeposition Zeit
 print8s stunde             ;Stunden
 print ':'                  ;Doppelpunkt
 print8s minute             ;Minuten
 print ':'                  ;Doppelpunkt
 print8s sekunde            ;Sekunden

 locate 3,2                 ;Position Akku-Meldung
 printf txtakku             ;Text,
 print8s batt1              ;Wert,
 print 32                   ;Leerzeichen
 print '%'                  ;Prozentzeichen
 print 32                   ;und noch'n Leerzeichen ausgeben
 rjmp mainloop              ;fertig...

messausgabe:            ;Messwerte ausgeben
 in wl,tcnt0                ;Zhlwert
 sts frequenz,wl            ;Counter0
 sts frequenz+1,tim0u1      ;mit bertrgen
 clr tim0u1                 ;
 out tcnt0,tim0u1
 sts frequenz+2,tim0u2      ;sichern
 clr tim0u2                 ;und lschen
 locate 1,0
 printf txtfreq
 locate 1,12                ;
 print24s frequenz,3,using  ;
 locate 2,0
 printf txtimpl
 locate 2,12                ;
 print24s impulsl,3,using   ;
 sts impulsl,null
 sts impulsl+1,null
 sts impulsl+2,null
 locate 3,0
 printf txtimph
 locate 3,12                ;
 print24s impulsh,3,using   ;
 sts impulsh,null
 sts impulsh+1,null
 sts impulsh+2,null
 
 rjmp mainloop              ;fertig...

akkutest:               ;Akku-Zustand messen (Vergleich Vcc mit V_bandgap)
 cbr flags,1<<hundertstel   ;Flag lschen, Job wird erledigt
 ldi wl,80                  ;Basiswert Akkuladezustand
 in wh,adch                 ;ADC einlesen (60...80)
 sub wl,wh                  ;(0...20)
 mov wh,wl                  ;Kopie 
 lsl wl                     ;mal
 lsl wl                     ;4,
 add wl,wh                  ;nochmal dazu entspricht mal 5
 lds r0,batt1               ;Mittelwert aus SRAM holen
 lds wh,batt0               ;Nachkommastellen auch
 sub wh,r0                  ;1/256 subtrahieren
 sbc r0,null                ;bertrag auch
 add wh,wl                  ;Neuwert/256 addieren
 adc r0,null                ;bertrag auch
 sts batt1,r0               ;zurck
 sts batt0,wh               ;ins SRAM
 rjmp mainloop              ;fertig...

halbsekunde:            ;Doppelpunkte lschen (Job der Mainloop)
 cbr flags,1<<neuhalb       ;Flag lschen (Job wird erledigt)
 sbis lcdstromport,lcdstrompin;Display aktiv? ja...
 rjmp mainloop              ;nein, Ausgabe unterdrcken...
 sbrs flags,uhraktiv        ;Uhr aktiv?
 rjmp mainloop              ;nein...

; rjmp mainloop              ;Debug, vorzeitig deaktiviert
 sbrc flags,uhrhalt         ;Uhr angehalten? nein...
 rjmp halbsekunde1          ;ja, nicht hochzhlen...
 locate 1,17                ;Doppelpunkte entfernen
 print ' '                  ;(sieht dann wie Blinken aus)
 locate 1,20
 print ' '
halbsekunde1:
 cpi mp,67                  ;Wochentag stellen?
 brne halbsekunde2          ;nein...
 locate 1,3                 ;Ausgabeposition Wochentagtext
 printf nixtag              ;Wochentag blinken lassen
halbsekunde2:
 cpi mp,68                  ;Stunde stellen?
 brne halbsekunde3          ;nein...
 locate 1,15                ;Ausgabeposition Stunde
 print ' '
 print ' '
halbsekunde3:
 cpi mp,69                  ;Minute stellen?
 brne halbsekunde4          ;nein...
 locate 1,18                ;Ausgabeposition Minute
 print ' '
 print ' '
halbsekunde4:
 cpi mp,70                  ;Sekunde stellen?
 brne halbsekunde5          ;nein...
 locate 1,21                ;Ausgabeposition Sekunde
 print ' '
 print ' '
halbsekunde5:
; locate 3,2                 ;Position
; in r10,adch                ;ADC einlesen
; print16 null,r10,0,fnul    ;und ausgeben
 ldi wl,(1<<adlar)+14       ;Bandgap linksbndig an
 out admux,wl               ;ADC legen
 rjmp mainloop


lcd_stromein:           ;schaltet Strom fr LCD ein
 sbi lcdstromport,lcdstrompin   ;Strom an und Port
 sbi lcdstromport-1,lcdstrompin ;als Ausgang
 rcall lcd_init                 ;LCD initialisieren
 ret

lcd_stromaus:           ;schaltet Strom fr LCD aus
;ret
 in wl,lcdport              ;LCD-Daten
 cbr wl,lcddatmsk           ;auf 0
 out lcdport,wl             ;setzen
 in wl,lcdport-1            ;LCD-Port
 cbr wl,lcddatmsk           ;auf Eingang
 out lcdport-1,wl           ;setzen
 cbi rsport,rs
 cbi rwport,rw
 cbi enable1port,enable1
 cbi enable2port,enable2
 cbi lcdstromport,lcdstrompin   ;Strom aus
 ret



menu:                   ;Auswerten der Tasten
 mov wh,mp                  ;
 andi wh,15                 ;Untermenpunkte aktiv?
 brne untermenu             ;ja...

 sbrc tfl,tre               ;rechte Taste? nein...
 rjmp menurechts            ;ja...

 sbrc tfl,tho               ;obere Taste? nein...
 subi mp,16                 ;1 Menebene zurck

 sbrc tfl,tru               ;untere Taste? nein...
 subi mp,-16                ;1 Menebene hoch

 cbr tfl,alltast            ;alle Tasten sind erledigt
 rjmp menujump              ;Menpunkt aufrufen...
untermenu:
 sbrc tfl,tre               ;rechte Taste? nein...
 inc mp                     ;ja, +1
 sbrc tfl,tli               ;linke Taste? nein...
 dec mp                     ;ja, -1
 cbr tfl,(1<<tre)|(1<<tli)  ;Tasten links und rechts sind erledigt
menujump:               ;Menpunktaufruf
 ldi zl,low(menujmp)        ;Zeiger auf Sprungtabelle
 ldi zh,high(menujmp)
 add zl,mp                  ;Menpunkt
 adc zh,null                ;dazu
 ijmp                       ;Menpunkt aufrufen
menurechts:
 inc mp                     ;Menpunkt um 1 hoch
 cbr tfl,alltast            ;alle Tasten sind erledigt
 rjmp menujump              ;weiter...

zur_uhr:                ;Bereichsbegrenzung
 ldi mp,32                  ;Menpunkt fr Uhr
 sbis lcdstromport,lcdstrompin;ist Display aktiv? ja...
 rcall lcd_stromein         ;nein, LCD aktivieren 

uhr:                    ;Menebene Uhr
 rcall lcd_clear            ;LCD lschen
 sbr flags,1<<uhraktiv      ;Zeit-Anzeige aktivieren
 locate 0,3                 ;Mentext
 printf mt_uhr              ;ausgeben 
 rjmp tastaus               ;zurck...

ibmess:                 ;Menebene Impulsbreitenmessung
 cbr flags,1<<uhraktiv      ;Zeitausgabe abschalten
 rcall lcd_clear            ;Display lschen
 locate 0,3                 ;Menpunkt
 printf mt_ibmess           ;ausgeben
 locate 1,0
 printf txtfreq
 locate 2,0
 printf txtimpl
 locate 3,0
 printf txtimph


ibmessend:
 rjmp tastaus               ;zurck...


weckzeit:               ;Menpunkt Weckzeit stellen

 rjmp tastaus               ;zurck...



uhrstellen0:            ;Menebene Uhr stellen, Punkt 0 
 rcall lcd_clear            ;LCD lschen
 sbr flags,1<<uhraktiv      ;Zeit-Anzeige aktivieren
 locate 0,3                 ;Mentext
 printf mt_64               ;ausgeben 
 rjmp tastaus               ;zurck...

uhrstellen1:            ;Menpunkt Uhr an/aus
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_65               ;ausgeben
 sbrc tfl,tho               ;Taste hoch? nein...
 cbr flags,1<<uhrhalt       ;Uhr einschalten
 sbrc tfl,tru               ;Taste runter? nein...
 sbr flags,1<<uhrhalt       ;Uhr ausschalten
 rjmp tastaus               ;zurck...

uhrstellen2:            ;Menpunkt Piepser an/aus
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_66               ;ausgeben
 sbrc tfl,tho               ;Taste hoch? nein...
 sbr flags,uhrpiep          ;Piepser einschalten
 sbrc tfl,tru               ;Taste runter? nein...
 cbr flags,uhrpiep          ;Piepser ausschalten
 rjmp tastaus               ;zurck...

uhrstellen3:            ;Wochentag stellen
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_67               ;ausgeben
 lds wl,wotag               ;Wochentag holen
 sbrs tfl,tho               ;Taste hoch? ja...
 rjmp uhrstellen3a          ;nein...
 inc wl                     ;Wochentag hoch
 cpi wl,7                   ;berlauf?
 brne uhrstellen3a          ;nein...
 clr wl                     ;ja
uhrstellen3a:
 sbrs tfl,tru               ;Taste runter? ja...
 rjmp uhrstellen3b          ;nein...
 dec wl                     ;Wochentag runter
 brpl uhrstellen3b          ;berlauf? nein...
 ldi wl,6                   ;ja, auf Startwert
uhrstellen3b:
 sts wotag,wl               ;Wochentag zurck ins SRAM
 locate 1,3                 ;Ausgabeposition Wochentagtext
 printts wochentag,wotag
 rjmp tastaus               ;zurck...

uhrstellen4:            ;Stunde stellen
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_68               ;ausgeben
 lds wl,stunde              ;Stunde holen
 sbrs tfl,tho               ;Taste hoch? ja...
 rjmp uhrstellen4a          ;nein...
 inc wl                     ;Stunde hoch
 cpi wl,24                  ;berlauf?
 brne uhrstellen4a          ;nein...
 clr wl                     ;ja
uhrstellen4a:
 sbrs tfl,tru               ;Taste runter? ja...
 rjmp uhrstellen4b          ;nein...
 dec wl                     ;Stunde runter
 brpl uhrstellen4b          ;berlauf? nein...
 ldi wl,23                   ;ja, auf Startwert
uhrstellen4b:
 sts stunde,wl              ;Stunde zurck ins SRAM
 locate 1,15                ;Ausgabeposition Stunden-Zehner
 print8s stunde
 rjmp tastaus               ;zurck...

uhrstellen5:            ;Minute stellen
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_69               ;ausgeben
 lds wl,minute              ;Minute holen
 sbrs tfl,tho               ;Taste hoch? ja...
 rjmp uhrstellen5a          ;nein...
 inc wl                     ;Minute hoch
 cpi wl,60                  ;berlauf?
 brne uhrstellen5a          ;nein...
 clr wl                     ;ja
uhrstellen5a:
 sbrs tfl,tru               ;Taste runter? ja...
 rjmp uhrstellen5b          ;nein...
 dec wl                     ;Minute runter
 brpl uhrstellen5b          ;berlauf? nein...
 ldi wl,59                  ;ja, auf Startwert
uhrstellen5b:
 sts minute,wl              ;Minute zurck ins SRAM
 locate 1,18                ;Ausgabeposition Minuten-Zehner
 print8s minute
 rjmp tastaus               ;zurck...

uhrstellen6:            ;Sekunde stellen
 rcall lcd_clear            ;LCD lschen
 locate 0,3                 ;Mentext
 printf mt_70               ;ausgeben
 lds wl,sekunde             ;Sekunde holen
 sbrs tfl,tho               ;Taste hoch? ja...
 rjmp uhrstellen6a          ;nein...
 inc wl                     ;Sekunde hoch
 cpi wl,60                  ;berlauf?
 brne uhrstellen6a          ;nein...
 clr wl                     ;ja
uhrstellen6a:
 sbrs tfl,tru               ;Taste runter? ja...
 rjmp uhrstellen6b          ;nein...
 dec wl                     ;Sekunde runter
 brpl uhrstellen6b          ;berlauf? nein...
 ldi wl,59                  ;ja, auf Startwert
uhrstellen6b:
 sts sekunde,wl             ;Sekunde zurck ins SRAM
 locate 1,21                ;Ausgabeposition Sekunde
 print8s sekunde
 rjmp tastaus               ;zurck...

uhrstellen7:
 cbr flags,1<<uhrhalt       ;Uhr einschalten
 rjmp zur_uhr

kalender:               ;Menpunkt Kalender stellen
 rcall lcd_clear            ;LCD lschen
 sbr flags,1<<uhraktiv      ;Zeit-Anzeige aktivieren
 locate 0,3                 ;Mentext
 printf mt_80               ;ausgeben 
 rjmp tastaus               ;zurck...





 rjmp tastaus               ;zurck...

lcdaus:                 ;Strom sparen, Display abschalten
 sbic lcdstromport,lcdstrompin  ;wenn Display an, dann
 rcall lcd_stromaus         ;Display deaktivieren
 andi tfl,(1<<tli)|(1<<tre) ;Taste hoch oder runter bettigt?
 breq lcdaus1               ;nein
 rjmp zur_uhr               ;ja, Display einschalten
lcdaus1:
 rjmp tastaus               ;zurck...


;----------------------------------------------------------------------------
menujmp:                ;Men-Sprungtabelle
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung

 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;
 rjmp ibmess                ;

 rjmp zur_uhr               ;
 rjmp lcdaus                ;Strom sparen, LCD aus
 rjmp zur_uhr               ;Dummy
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;
 rjmp uhr                   ;

 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;
 rjmp weckzeit              ;

 rjmp uhrstellen0           ;
 rjmp uhrstellen1           ;
 rjmp uhrstellen2           ;
 rjmp uhrstellen3           ;
 rjmp uhrstellen4           ;
 rjmp uhrstellen5           ;
 rjmp uhrstellen6           ;
 rjmp uhrstellen7           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;
 rjmp uhrstellen0           ;

 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;
 rjmp kalender              ;

 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung
 rjmp zur_uhr               ;Bereichsbegrenzung


Uhrdaten:
.db "Zeit:   12:00:00",0,0

zeit:
.db "Zeit:   ",0,0
copyright:
.db "Copyleft 02/2005",0,0
hannes:
.db "by ...HanneS...",0

wochentag:  ;Tabelle auf Wochentage
.dw sonntag
.dw montag
.dw dienstag
.dw mittwoch
.dw donnerstag
.dw freitag
.dw samstag
.dw nixtag

sonntag:    .db "Sonntag    ",0
montag:     .db "Montag     ",0
dienstag:   .db "Dienstag   ",0
mittwoch:   .db "Mittwoch   ",0
donnerstag: .db "Donnerstag ",0
freitag:    .db "Freitag    ",0
samstag:    .db "Sonnabend  ",0
nixtag:     .db "           ",0
zweileer:   .db "  ",0,0

mt_ibmess:  .db "Impulsbreitenmessung   ",0
mt_uhr:     .db "Uhr mit Wochentag...   ",0

mt_64:      .db "Uhrzeit stellen...     ",0
mt_65:      .db "Start / Stop der Uhr   ",0
mt_66:      .db "Summer ein / aus       ",0
mt_67:      .db "Wochentag stellen + / -",0
mt_68:      .db "Stunde stellen + / -   ",0
mt_69:      .db "Minute stellen + / -   ",0
mt_70:      .db "Sekunde stellen + / -  ",0

mt_80:      .db "Kalender stellen...    ",0


txtfreq:    .db "Frequenz:            kHz",0,0
txtimpl:    .db "Impulspause:         ms ",0,0
txtimph:    .db "Impulsdauer:         ms ",0,0

txtakku:    .db "Akku-Ladezustand: ",0,0

txtmp:      .db "Men",$f5,"punkt: ",0

ze0:        .db "Zeile 0",0
ze1:        .db "Zeile 1",0
ze2:        .db "Zeile 2",0
ze3:        .db "Zeile 3",0
ze4:        .db "Zeile 4",0
ze5:        .db "Zeile 5",0
ze6:        .db "Zeile 6",0
ze7:        .db "Zeile 7",0
